% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/generateSimulations.R
\docType{methods}
\name{generateSimulation,dgCMatrix,numeric,numeric,numeric,numeric,numeric,numeric-method}
\alias{generateSimulation,dgCMatrix,numeric,numeric,numeric,numeric,numeric,numeric-method}
\title{generate a diffrential chromatin simulation}
\usage{

  \S4method{generateSimulation}{dgCMatrix,numeric,numeric,numeric,numeric,numeric,numeric}(baseHic = NULL,
  pUp, foldDiff = 20, pDiff = 0.01, nrep1 = 2, nrep2 = 2,
  extreme.dispersion = 0.1)
}
\arguments{
\item{baseHic}{a dgCMatrix used a base to generate the simulated
diffrential Hi-C matrices. The values in the \Rcode{baseHic} represent
the interaction means.}

\item{pUp}{numeric. Probability that a diffrential interaction will show an
 increase in frequency in the second condition.
The probability to show a decrease is equal to \Rcode{1-pUp}.}

\item{foldDiff}{double. The amout of fold change.}

\item{pDiff}{double. The probability that an interaction will be diffrential.}

\item{nrep1}{numeric. The number of replicates in the first condition.}

\item{nrep2}{numeric. The number of replicates in the second condition.}

\item{extreme.dispersion}{double. Remove interactions that show
a dispersion value larger than this value.}
}
\value{
a list that contains the simulates replicates and the
matrix of the true DCI regions.
The list contains the following elements:
Hic1 : A list containing the simulated Hi-C matrices of the first condition.
Hic2 : A list containing the simulated Hi-C matrices of the second condition.
trueStates : A sparceMatric containing the position of the diffrential interactions.
}
\description{
This function uses a binomial distribution to generate the counts of the
simulated samples.
Additionally, for the differentially intracting regions, we use a gaussian
smoothing function to elumate the effect of neighbors change.
The implementation of this function is mainly based on the code by at :
 http://130.60.190.4/robinson_lab/edgeR_robust/robust_simulation.R
with modifications to add the gaussian smoothing part.
}
\examples{
\dontrun{
require(gridExtra)
data(K562_matrix_small)
simRes <- generateSimulation(K562_matrix_small[1:300,1:300],pDiff = 0.01,pUp = 0.4,
                             foldDiff = 5,extreme.dispersion = 0.1,
                             nrep1=2,nrep2=2)
# Display the simulated data
p1 <- levelplot(as.matrix(simRes$Hic1[[1]]), par.settings=YlOrRdTheme,main="Cond1 Rep1")
p2 <- levelplot(as.matrix(simRes$Hic1[[2]]), par.settings=YlOrRdTheme,main="Cond1 Rep2")
p3 <- levelplot(as.matrix(simRes$Hic2[[1]]), par.settings=YlOrRdTheme,main="Cond2 Rep1")
p4 <- levelplot(as.matrix(simRes$Hic2[[2]]), par.settings=YlOrRdTheme,main="Cond2 Rep2")

#Plot the position of the diffrentlly interacting regions
p5 <- levelplot(as.matrix(simRes$trueStates), par.settings= YlOrRdTheme, main="DCIs position")
## Plot the log2 Fold-change between the two conditions
m1 <- simRes$Hic1[[1]] + simRes$Hic1[[2]]
m2 <- simRes$Hic2[[1]] + simRes$Hic2[[2]]
p6 <- levelplot(as.matrix(log2((m1+0.1)/(m2+0.1))), par.settings = BuRdTheme, main="log2(Cond1/cond2)")
grid.arrange(p1,p2,p3,p4,p5,p6, ncol=3)
}
}
